@@ -20,9 +20,6 @@ class DateFormat(StrEnum):
     COMPACT = "%Y%m%d"
     INV = "%Y-%m-%d"
 
-    def __str__(self) -> str:  # noqa: D105
-        return self.name
-
 
 class DatetimeFormat(StrEnum):
     """
@@ -33,7 +30,8 @@ class DatetimeFormat(StrEnum):
     INV = "%Y-%m-%d %H:%M:%S"
 
     def __str__(self) -> str:  # noqa: D105
-        return self.name
+        # noinspection PyTypeChecker
+        return self.value
 
 
 def date_reformat(dt_str: str,
