@@ -1,30 +1,33 @@
 from datetime import date, datetime
+from flask import jsonify, Response
 from logging import Logger
 from typing import Any, Final
 from .datetime_pomes import TIMEZONE_LOCAL
 from .env_pomes import APP_PREFIX, env_get_str
-from .str_pomes import str_sanitize, str_find_whitespace
+from .str_pomes import str_as_list, str_sanitize, str_find_whitespace
 
-VALIDATION_MSG_LANGUAGE: Final[str] = env_get_str(f"{APP_PREFIX}_VALIDATION_MSG_LANGUAGE", "en")
-VALIDATION_MSG_PREFIX: Final[str] = env_get_str(f"{APP_PREFIX}_VALIDATION_MSG_PREFIX", APP_PREFIX)
+VALIDATION_MSG_LANGUAGE: Final[str] = env_get_str(key=f"{APP_PREFIX}_VALIDATION_MSG_LANGUAGE",
+                                                  def_value="en")
+VALIDATION_MSG_PREFIX: Final[str] = env_get_str(key=f"{APP_PREFIX}_VALIDATION_MSG_PREFIX",
+                                                def_value=APP_PREFIX)
 
 
 def validate_value(attr: str,
                    val: str | int | float,
                    min_val: int = None,
                    max_val: int = None,
-                   values: list[Any] = None,
+                   values: list = None,
                    required: bool = False) -> str:
     """
     Validate *val* according to value, range, or membership in values list, as specified.
 
     :param attr: the name of the attribute
     :param val: the value to be validated
-    :param min_val: if val is a string, specifies its minimum length; otherwise, specifies its minimum value
-    :param max_val: if val is a string, specifies its maximum length; otherwise, specifies its maximum value
-    :param values: if provided, requires 'val' to be in it
-    :param required:  requires 'val' to be specified;
-    :return: 'None' if val passes validation, or the corresponding error message otherwise
+    :param min_val: if *val* is a string, specifies its minimum length; otherwise, specifies its minimum value
+    :param max_val: if *val* is a string, specifies its maximum length; otherwise, specifies its maximum value
+    :param values: if provided, requires *val* to be contained therein
+    :param required:  requires *val* to be specified
+    :return: *None* if *val* passes validation, or the corresponding error message otherwise
     """
     # initialize the return variable
     result: str | None = None
@@ -86,10 +89,10 @@ def validate_bool(errors: list[str] | None,
     :param errors: incidental error messages
     :param scheme: dictionary containing the value to be validated
     :param attr: the name of the attribute whose value is being validated
-    :param default: default value, overiding 'required'
+    :param default: default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or 'None' if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # initialize the return variable
     result: bool | None = None
@@ -142,7 +145,8 @@ def validate_int(errors: list[str] | None,
                  attr: str,
                  min_val: int = None,
                  max_val: int = None,
-                 default: int | list[int] = None,
+                 values: list[int] = None,
+                 default: int = None,
                  required: bool = False,
                  logger: Logger = None) -> int:
     """
@@ -155,11 +159,11 @@ def validate_int(errors: list[str] | None,
     :param attr: the attribute associated with the value to be validated
     :param min_val: the minimum value accepted
     :param max_val:  the maximum value accepted
-    :param default: if 'int', specifies the default value, overriding the other constraints
-                    if 'list', requires the value, if provided, to be in it
+    :param values: optional list of allowed values
+    :param default: optional default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or 'None' if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # initialize the return variable
     result: int | None = None
@@ -171,19 +175,17 @@ def validate_int(errors: list[str] | None,
     # retrieve the value
     value: int = scheme.get(suffix)
 
-    # validate it
-    if value is None and isinstance(default, int):
+    # validate it ('bool' is subtype of 'int')
+    if value is None and isinstance(default, int) and not isinstance(default, bool):
         value = default
     elif isinstance(value, str) and value.isnumeric():
         value = int(value)
-    # 'bool' is subtype of 'int'
     elif value is not None and \
-         (isinstance(value, bool) or not isinstance(value, int)):
+            (isinstance(value, bool) or not isinstance(value, int)):
         # 152: Invalid value {}: must be type {}
         stat = validate_format_error(152, value, "int", f"@{attr}")
 
     if not stat:
-        values = default if isinstance(default, list) else None
         stat = validate_value(attr=attr,
                               val=value,
                               min_val=min_val,
@@ -206,7 +208,8 @@ def validate_float(errors: list[str] | None,
                    min_val: float = None,
                    max_val: float = None,
                    required: bool = False,
-                   default: int | float | list[float | int] = None,
+                   values: list[float | int] = None,
+                   default: int | float = None,
                    logger: Logger = None) -> float:
     """
     Validate the *float* value associated with *attr* in *scheme*.
@@ -218,11 +221,11 @@ def validate_float(errors: list[str] | None,
     :param attr: the attribute associated with the value to be validated
     :param min_val: the minimum value accepted
     :param max_val:  the maximum value accepted
-    :param default: if 'float' or 'int', specifies the default value, overriding the other constraints
-                    if 'list', requires the value, if provided, to be in it
+    :param values: optional list of allowed values
+    :param default: optional default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or 'None' if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # initialize the return variable
     result: float | None = None
@@ -238,15 +241,14 @@ def validate_float(errors: list[str] | None,
     if value is None and isinstance(default, int | float):
         value = float(default)
     elif (isinstance(value, int) and not isinstance(value, bool)) or \
-          (isinstance(value, str) and value.replace(".", "", 1).isnumeric()):
+            (isinstance(value, str) and value.replace(".", "", 1).isnumeric()):
         value = float(value)
     elif isinstance(value, bool) or \
-         (value is not None and not isinstance(value, int | float)):
+            (value is not None and not isinstance(value, int | float)):
         # 152: Invalid value {}: must be type {}
         stat = validate_format_error(152, value, "float", f"@{attr}")
 
     if not stat:
-        values = default if isinstance(default, list) else None
         stat = validate_value(attr=attr,
                               val=value,
                               min_val=min_val,
@@ -268,7 +270,8 @@ def validate_str(errors: list[str] | None,
                  attr: str,
                  min_length: int = None,
                  max_length: int = None,
-                 default: str | list[str] = None,
+                 values: list[str] = None,
+                 default: str = None,
                  required: bool = False,
                  logger: Logger = None) -> str:
     """
@@ -281,11 +284,11 @@ def validate_str(errors: list[str] | None,
     :param attr: the attribute associated with the value to be validated
     :param min_length: optional minimum length accepted
     :param max_length:  optional maximum length accepted
-    :param default: if 'str', specifies the default value, overrinding the other constraints
-                    if 'list', requires the value, if provided, to be in it
+    :param values: optional list of allowed values
+    :param default: optional default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or None if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # initialize the return variable
     result: str | None = None
@@ -304,14 +307,12 @@ def validate_str(errors: list[str] | None,
         # 152: Invalid value {}: must be type {}
         stat = validate_format_error(152, value, "str", f"@{attr}")
     else:
-        values = default if isinstance(default, list) else None
         stat = validate_value(attr=attr,
                               val=value,
                               min_val=min_length,
                               max_val=max_length,
                               values=values,
-                              required= required)
-
+                              required=required)
     if stat:
         __validate_log(errors=errors,
                        err_msg=stat,
@@ -338,10 +339,10 @@ def validate_date(errors: list[str] | None,
     :param scheme: dictionary containing the value to be validated
     :param attr: the attribute associated with the value to be validated
     :param day_first: indicates that the day precedes the month in the string representing the date
-    :param default: optional default value, overrides 'required'
+    :param default: optional default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or 'None' if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # import needed module
     from .datetime_pomes import date_parse
@@ -377,7 +378,6 @@ def validate_date(errors: list[str] | None,
         __validate_log(errors=errors,
                        err_msg=stat,
                        logger=logger)
-
     return result
 
 
@@ -397,10 +397,10 @@ def validate_datetime(errors: list[str] | None,
     :param scheme: dictionary containing the value to be validated
     :param attr: the attribute associated with the value to be validated
     :param day_first: indicates that the day precedes the month in the string representing the date
-    :param default: optional default value, overrides 'required'
+    :param default: optional default value, overrides *required*
     :param required: specifies whether a value must be provided
     :param logger: optional logger
-    :return: the validated value, or 'None' if validation failed
+    :return: the validated value, or *None* if validation failed
     """
     # import needed module
     from .datetime_pomes import datetime_parse
@@ -433,7 +433,6 @@ def validate_datetime(errors: list[str] | None,
         __validate_log(errors=errors,
                        err_msg=stat,
                        logger=logger)
-
     return result
 
 
@@ -447,7 +446,7 @@ def validate_ints(errors: list[str] | None,
     """
     Validate the list of *int* values associated with *attr* in *scheme*.
 
-    If provided, this list must contain *ints*, or valid string representations of *ints*.
+    If provided, this list must contain *int*s, or valid string representations of *int*s.
 
     :param errors: incidental error messages
     :param scheme: dictionary containing the list of values to be validated
@@ -456,18 +455,20 @@ def validate_ints(errors: list[str] | None,
     :param max_val:  the maximum value accepted
     :param required: whether the list of values must be provided
     :param logger: optional logger
-    :return: the list of validated values, '[]' if not required and no values found, or 'None' if validation failed
+    :return: the list of validated values, *[]* if not required and no values found, or *None* if validation failed
     """
     # initialize the return variable
-    result: list[Any] | None = []
+    result: list | None = []
 
     stat: str | None = None
     pos: int = attr.rfind(".") + 1
     suffix: str = attr[pos:]
 
     # obtain the values
-    values: list[Any] = scheme.get(suffix)
+    values: list = scheme.get(suffix)
     if values:
+        if isinstance(values, str):
+            values = str_as_list(values)
         if isinstance(values, list):
             result = []
             if len(values) > 0:
@@ -510,7 +511,7 @@ def validate_strs(errors: list[str] | None,
     """
     Validate the list of *str* values associated with *attr* in *scheme*.
 
-    If provided, this list must contain *strs*.
+    If provided, this list must contain *str*s.
 
     :param errors: incidental error messages
     :param scheme: dictionary containing the list of values to be validated
@@ -519,18 +520,20 @@ def validate_strs(errors: list[str] | None,
     :param max_length:  optional maximum length accepted
     :param required: whether the list of values must be provided
     :param logger: optional logger
-    :return: the list of validated values, '[]' if not required and no values found, or 'None' if validation failed
+    :return: the list of validated values, *[]* if not required and no values found, or *None* if validation failed
     """
     # initialize the return variable
-    result: list[Any] | None = []
+    result: list | None = []
 
     stat: str | None = None
     pos: int = attr.rfind(".") + 1
     suffix: str = attr[pos:]
 
     # obtain the values
-    values: list[Any] = scheme.get(suffix)
+    values: list = scheme.get(suffix)
     if values:
+        if isinstance(values, str):
+            values = str_as_list(values)
         if isinstance(values, list):
             result = []
             if len(values) > 0:
@@ -563,6 +566,31 @@ def validate_strs(errors: list[str] | None,
     return result
 
 
+def validate_build_response(errors: list[str],
+                            reply: dict) -> Response:
+    """
+    Build a *Response* object based on the given *errors* list and the set of key/value pairs in *reply*.
+
+    :param errors: the reference errors
+    :param reply: the key/value pairs to add to the response as JSON string
+    :return: the appropriate *Response* object
+    """
+    # declare the return variable
+    result: Response
+
+    if len(errors) == 0:
+        # 'reply' might be 'None'
+        result = jsonify(reply)
+    else:
+        reply_err: dict = {"errors": validate_format_errors(errors=errors)}
+        if isinstance(reply, dict):
+            reply_err.update(reply)
+        result = jsonify(reply_err)
+        result.status_code = 400
+
+    return result
+
+
 def validate_format_error(error_id: int,
                           *args: Any) -> str:
     """
@@ -622,7 +650,7 @@ def validate_format_errors(errors: list[str]) -> list[dict[str, str]]:
     :return: the built list
     """
     # initialize the return variable
-    result: list[dict] = []
+    result: list[dict[str, str]] = []
 
     # extract error code, description, and attribute from text
     for error in errors:
